@@ -4,7 +4,7 @@ package JSON::Schema::Modern::Document;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: One JSON Schema document
 
-our $VERSION = '0.571';
+our $VERSION = '0.572';
 
 use 5.020;
 use Moo;
@@ -21,7 +21,6 @@ use List::Util 1.29 'pairs';
 use Ref::Util 0.100 'is_plain_hashref';
 use Safe::Isa 1.000008;
 use MooX::TypeTiny;
-use MooX::HandlesVia;
 use Types::Standard 1.016003 qw(InstanceOf HashRef Str Dict ArrayRef Enum ClassName Undef Slurpy);
 use namespace::clean;
 
@@ -65,32 +64,26 @@ has resource_index => (
       configs => HashRef,
       Slurpy[HashRef[Undef]],  # no other fields allowed
     ]],
-  handles_via => 'Hash',
-  handles => {
-    resource_index => 'elements',
-    resource_pairs => 'kv',
-    _add_resources => 'set',
-    _get_resource => 'get',
-    _remove_resource => 'delete',
-    _canonical_resources => 'values',
-  },
   init_arg => undef,
-  lazy => 1,
   default => sub { {} },
 );
 
+sub resource_index { $_[0]->{resource_index}->%* }
+sub resource_pairs { pairs $_[0]->{resource_index}->%* }
+sub _add_resources { $_[0]->{resource_index}{$_[1]} = $resource_type->($_[2]) }
+sub _get_resource { $_[0]->{resource_index}{$_[1]} }
+sub _canonical_resources { values $_[0]->{resource_index}->%* }
+
 has _path_to_resource => (
-  is => 'bare',
+  is => 'ro',
   isa => HashRef[$resource_type],
-  handles_via => 'Hash',
-  handles => {
-    path_to_resource => 'get',
-  },
   init_arg => undef,
   lazy => 1,
   default => sub { +{ map +($_->{path} => $_), shift->_canonical_resources } },
 );
 
+sub path_to_resource { $_[0]->_path_to_resource; $_[0]->{_path_to_resource}{$_[1]} }
+
 # for internal use only
 has _serialized_schema => (
   is => 'rw',
@@ -100,17 +93,15 @@ has _serialized_schema => (
 
 has errors => (
   is => 'bare',
-  handles_via => 'Array',
-  handles => {
-    errors => 'elements',
-    has_errors => 'count',
-  },
   writer => '_set_errors',
   isa => ArrayRef[InstanceOf['JSON::Schema::Modern::Error']],
   lazy => 1,
   default => sub { [] },
 );
 
+sub errors { ($_[0]->{errors}//[])->@* }
+sub has_errors { scalar ($_[0]->{errors}//[])->@* }
+
 around _add_resources => sub {
   my $orig = shift;
   my $self = shift;
@@ -142,9 +133,10 @@ sub FOREIGNBUILDARGS { () }
 # for JSON serializers
 sub TO_JSON { shift->schema }
 
+# note that this is always called, even in subclasses
 sub BUILD ($self, $args) {
   my $original_uri = $self->canonical_uri->clone;
-  my $state = $self->traverse($self->evaluator // JSON::Schema::Modern->new);
+  my $state = $self->traverse($self->evaluator // $self->_set_evaluator(JSON::Schema::Modern->new));
 
   # if the schema identified a canonical uri for itself, it overrides the initial value
   $self->_set_canonical_uri($state->{initial_schema_uri}) if $state->{initial_schema_uri} ne $original_uri;
@@ -172,6 +164,7 @@ sub BUILD ($self, $args) {
   $self->_add_resources($state->{identifiers}->@*);
 }
 
+# a subclass's method will override this one
 sub traverse ($self, $evaluator) {
   die 'wrong class - use JSON::Schema::Modern::Document::OpenAPI instead'
     if is_plain_hashref($self->schema) and exists $self->schema->{openapi};
@@ -217,7 +210,7 @@ JSON::Schema::Modern::Document - One JSON Schema document
 
 =head1 VERSION
 
-version 0.571
+version 0.572
 
 =head1 SYNOPSIS
 
@@ -287,7 +280,7 @@ errors halt the parsing process.) Documents with errors cannot be evaluated.
 
 =head1 METHODS
 
-=for Pod::Coverage FOREIGNBUILDARGS BUILDARGS BUILD traverse
+=for Pod::Coverage FOREIGNBUILDARGS BUILDARGS BUILD traverse has_errors path_to_resource resource_pairs
 
 =head2 path_to_canonical_uri
 
