@@ -4,7 +4,7 @@ package JSON::Schema::Modern::Result;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Contains the result of a JSON Schema evaluation
 
-our $VERSION = '0.571';
+our $VERSION = '0.572';
 
 use 5.020;
 use Moo;
@@ -17,7 +17,6 @@ no if "$]" >= 5.033001, feature => 'multidimensional';
 no if "$]" >= 5.033006, feature => 'bareword_filehandles';
 use MooX::TypeTiny;
 use Types::Standard qw(ArrayRef InstanceOf Enum Bool);
-use MooX::HandlesVia;
 use JSON::Schema::Modern::Annotation;
 use JSON::Schema::Modern::Error;
 use JSON::PP ();
@@ -53,11 +52,6 @@ has $_.'s' => (
   isa => ArrayRef[InstanceOf['JSON::Schema::Modern::'.ucfirst]],
   lazy => 1,
   default => sub { [] },
-  handles_via => 'Array',
-  handles => {
-    $_.'s' => 'elements',
-    $_.'_count' => 'count',
-  },
   coerce => do {
     my $type = $_;
     sub ($arrayref) {
@@ -67,6 +61,11 @@ has $_.'s' => (
   },
 ) foreach qw(error annotation);
 
+sub errors { ($_[0]->{errors}//[])->@* }
+sub error_count { scalar ($_[0]->{errors}//[])->@* }
+sub annotations { ($_[0]->{annotations}//[])->@* }
+sub annotation_count { scalar ($_[0]->{annotations}//[])->@* }
+
 # strict_basic can only be used with draft2019-09.
 use constant OUTPUT_FORMATS => [qw(flag basic strict_basic detailed verbose terse data_only)];
 
@@ -218,7 +217,7 @@ JSON::Schema::Modern::Result - Contains the result of a JSON Schema evaluation
 
 =head1 VERSION
 
-version 0.571
+version 0.572
 
 =head1 SYNOPSIS
 
@@ -319,7 +318,7 @@ A boolean flag indicating whether L</format> should include annotations in the o
 
 =head1 METHODS
 
-=for Pod::Coverage BUILD OUTPUT_FORMATS result stringify
+=for Pod::Coverage BUILD OUTPUT_FORMATS result stringify annotation_count error_count
 
 =head2 format
 
