@@ -4,7 +4,7 @@ package JSON::Schema::Modern::Vocabulary::FormatAssertion;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Implementation of the JSON Schema Format-Assertion vocabulary
 
-our $VERSION = '0.571';
+our $VERSION = '0.572';
 
 use 5.020;
 use Moo;
@@ -208,7 +208,7 @@ sub _eval_keyword_format ($self, $data, $schema, $state) {
     }
   }
   catch ($e) {
-    abort($state, 'EXCEPTION: cannot validate format "%s": %s', $schema->{format}, $e);
+    abort($state, 'EXCEPTION: cannot validate with format "%s": %s', $schema->{format}, $e);
   }
 
   # first check the subrefs from JSON::Schema::Modern->new(format_validations => { ... })
@@ -242,7 +242,7 @@ JSON::Schema::Modern::Vocabulary::FormatAssertion - Implementation of the JSON S
 
 =head1 VERSION
 
-version 0.571
+version 0.572
 
 =head1 DESCRIPTION
 
